#include <errno.h> /* EACCESS, ELOOP, ENAMETOOLONG, ENOENT, ENOTDIR, errno */
#include <stdio.h> /* fprintf(3) */
#include <stdlib.h> /* stderr */
#include <sysexits.h> /* EX_OSERR, EX_NOPERM, EX_NOINPUT, EX_UNAVAILABLE,
                       * EX_USAGE */
#include <unistd.h> /* chdir(2) */

static char *program_name = "pschdir";

int main(int argc, char *argv[]){

	if(argc < 3){
		fprintf(stderr,
			"Usage: %s [directory] [command (argument...)]\n",
			argv[0] == NULL ? program_name : argv[0]);
		return EX_USAGE;
	}

	if(chdir(argv[1]) != 0){
		fprintf(stderr, "%s: %s: ", argv[0], argv[1]);
		switch(errno){
		case EACCES:
			fprintf(stderr, "Access to directory denied.\n");
			return EX_NOPERM;
		case ELOOP:
			fprintf(stderr, "Symbolic link loop.\n");
			return EX_UNAVAILABLE;
		case ENAMETOOLONG:
			fprintf(stderr, "Name too long.\n");
			return EX_OSERR;
		case ENOENT:
			fprintf(stderr, "Not found.\n");
			return EX_NOINPUT;
		case ENOTDIR:
			fprintf(stderr, "Not a directory.\n");
			return EX_NOINPUT;
		default:
			fprintf(stderr, "Unknown error.\n");
			return EX_UNAVAILABLE;
		}
	}

	execvp(argv[2], argv + 2);
}
